@@ -38,7 +38,7 @@ class Stub(object):
         '''Creates a fresh class per instance. This is required due to
         ad-hoc stub properties are class attributes'''
         klass = type(cls.__name__, (cls,), dict(cls.__dict__))
-        return object.__new__(klass, collaborator)
+        return object.__new__(klass)
 
     def __init__(self, collaborator=None):
         self._proxy = create_proxy(collaborator)
@@ -61,15 +61,22 @@ class Stub(object):
             self._stubs.append(invocation)
             return invocation
 
-        self._do_manage_invocation(invocation)
+        self._prepare_invocation(invocation)
 
+        have_retval = False
         if invocation in self._stubs:
             stubbed = self._stubs.lookup(invocation)
-            return stubbed.perform(invocation)
+            retval = stubbed.perform(invocation)
+            have_retval = True
 
-        return self._perform_invocation(invocation)
+        actual_retval = self._perform_invocation(invocation)
 
-    def _do_manage_invocation(self, invocation):
+        if not have_retval:
+            return actual_retval
+
+        return retval
+
+    def _prepare_invocation(self, invocation):
         pass
 
     def _perform_invocation(self, invocation):
@@ -80,14 +87,17 @@ class Stub(object):
         return object.__getattribute__(self, key)
 
     def _add_attr(self, key):
-        attr = AttributeFactory.create(self, key)
-        setattr(self.__class__, key, attr)
+        setattr(self.__class__, key, AttributeFactory.create(self, key))
 
     def __setattr__hook(self, key, value):
         if key in self.__dict__:
             return object.__setattr__(self, key, value)
 
-        self._add_attr(key)
+        try:
+            self._add_attr(key)
+        except AttributeError:
+            pass
+
         object.__setattr__(self, key, value)
 
     def _classname(self):
@@ -100,16 +110,11 @@ class Spy(Stub, SpyBase):
         self._recorded = OperationList()
         super(Spy, self).__init__(collaborator)
 
-    def _do_manage_invocation(self, invocation):
+    def _prepare_invocation(self, invocation):
         self._recorded.append(invocation)
 
-    def _was_called(self, invocation, times):
-        try:
-            hamcrest.assert_that(self._recorded.count(invocation),
-                                 hamcrest.is_(times))
-            return True
-        except AssertionError:
-            return False
+    def _received_invocation(self, invocation, times):
+        return hamcrest.is_(times).matches(self._recorded.count(invocation))
 
     def _get_invocations_to(self, name):
         return [i for i in self._recorded
@@ -118,10 +123,10 @@ class Spy(Stub, SpyBase):
 
 class ProxySpy(Spy):
     def __init__(self, collaborator):
-        self._assure_instance(collaborator)
+        self._assure_is_instance(collaborator)
         super(ProxySpy, self).__init__(collaborator)
 
-    def _assure_instance(self, thing):
+    def _assure_is_instance(self, thing):
         if thing is None or inspect.isclass(thing):
             raise TypeError("ProxySpy takes an instance (got %s instead)" % thing)
 
@@ -130,9 +135,9 @@ class ProxySpy(Spy):
 
 
 class Mock(Spy, MockBase):
-    def _do_manage_invocation(self, invocation):
+    def _prepare_invocation(self, invocation):
         hamcrest.assert_that(self, MockExpectInvocation(invocation))
-        super(Mock, self)._do_manage_invocation(invocation)
+        super(Mock, self)._prepare_invocation(invocation)
 
 
 def Mimic(double, collab):
@@ -146,8 +151,9 @@ def Mimic(double, collab):
         return self._get_method(key)
 
     def _get_method(self, key):
-        if key not in self._methods.keys():
-            assert self._proxy.get_attr_typeid(key) == 'instancemethod'
+        if key not in list(self._methods.keys()):
+            typename = self._proxy.get_attr_typename(key)
+            assert typename in ['instancemethod', 'function', 'method'], typename
             method = Method(self, key)
             self._methods[key] = method
 
