@@ -24,7 +24,7 @@ import inspect
 try:
     from inspect import getcallargs
 except ImportError:
-    from py27_backports import getcallargs
+    from .py27_backports import getcallargs
 
 from .internal import ANY_ARG
 
@@ -37,7 +37,7 @@ def create_proxy(collaborator):
 
 
 class DummyProxy(object):
-    def get_attr_typeid(self, key):
+    def get_attr_typename(self, key):
         return 'instancemethod'
 
     def assure_signature_matches(self, invocation):
@@ -61,42 +61,33 @@ class Proxy(object):
     '''Represent the collaborator object'''
     def __init__(self, collaborator):
         self.collaborator = collaborator
-        self.collaborator_class = self.get_class()
+        self.collaborator_class = get_class(collaborator)
 
     def isclass(self):
         return inspect.isclass(self.collaborator)
 
-    def get_class(self):
-        if self.isclass():
-            return self.collaborator
-        else:
-            return self.collaborator.__class__
-
     def get_class_attr(self, key):
         return getattr(self.collaborator_class, key)
 
     def get_attr(self, key):
         return getattr(self.collaborator, key)
 
-    def get_attr_typeid(self, key):
-        try:
-            return type(getattr(self.collaborator, key)).__name__
-        except AttributeError:
-            reason = "'%s' object has no attribute '%s'" % \
-                (self.collaborator_classname(), key)
-            raise AttributeError(reason)
-
     def collaborator_classname(self):
         return self.collaborator_class.__name__
 
     def assure_signature_matches(self, invocation):
-        assert self.get_attr_typeid(invocation.name) == 'instancemethod'
-        signature = Signature(self, invocation.name)
+        signature = create_signature(self, invocation.name)
+        signature.assure_match(invocation.context.args,
+                               invocation.context.kargs)
+
+    def get_attr_typename(self, key):
         try:
-            signature.assure_match(invocation.context.args,
-                                   invocation.context.kargs)
-        except TypeError, e:
-            raise TypeError("%s.%s" % (self.get_class(), e))
+            attr = getattr(self.collaborator, key)
+            return type(attr).__name__
+        except AttributeError:
+            reason = "'%s' object has no attribute '%s'" % \
+                (self.collaborator_classname(), key)
+            raise AttributeError(reason)
 
     def same_method(self, name1, name2):
         return getattr(self.collaborator, name1) == \
@@ -108,35 +99,49 @@ class Proxy(object):
                        **invocation.context.kargs)
 
 
+def create_signature(proxy, method_name):
+    method = getattr(proxy.collaborator, method_name)
+    if not is_method_or_func(method):
+        return BuiltinSignature(proxy, method_name)
+
+    return Signature(proxy, method_name)
+
+
+def is_method_or_func(func):
+    if inspect.ismethod(func):
+        func = func.im_func
+    return inspect.isfunction(func)
+
+
+class BuiltinSignature(object):
+    "builtin collaborator method signature"
+    def __init__(self, proxy, name):
+        self.proxy = proxy
+        self.name = name
+        self.method = getattr(proxy.collaborator, name)
+
+    def assure_match(self, args, kargs):
+        doc = self.method.__doc__
+        if not ')' in doc:
+            return
+
+        rpar = doc.find(')')
+        params = doc[:rpar]
+        nkargs = params.count('=')
+        nargs = params.count(',') + 1 - nkargs
+        if len(args) != nargs:
+            raise TypeError('%s.%s() takes exactly %s argument (%s given)' % (
+                    self.proxy.collaborator_classname(), self.name, nargs, len(args)))
+
+
 class Signature(object):
-    """colaborator method signature"""
+    "colaborator method signature"
     def __init__(self, proxy, name):
         self.proxy = proxy
         self.name = name
         self.method = getattr(proxy.collaborator, name)
         self.argspec = inspect.getargspec(self.method)
 
-#        print "signature:", self.method
-#        print "class:    ", self.method.__class__
-#        print "args(%s):   %s" % (len(self.argspec.args), self.argspec.args)
-#        print "varargs:  ", self.argspec.varargs
-#        print "keywords: ", self.argspec.keywords
-#        print "defaults: ", self.argspec.defaults
-
-#    def count_positionals(self):
-#        if self.argspec.defaults is None:
-#            ndefaults = 0
-#        else:
-#            ndefaults = len(self.argspec.defaults)
-#
-#        return len(self.argspec.args) - ndefaults - 1
-#
-#    def get_keywords(self):
-#        if self.argspec.defaults is None:
-#            return []
-#
-#        return self.argspec.args[-len(self.argspec.defaults):]
-
     def assure_match(self, args, kargs):
         if ANY_ARG in args:
             return
@@ -144,7 +149,10 @@ class Signature(object):
         if self.proxy.isclass():
             args = (None,) + args  # self
 
-        getcallargs(self.method, *args, **kargs)
+        try:
+            getcallargs(self.method, *args, **kargs)
+        except TypeError as e:
+            raise TypeError("%s.%s" % (self.proxy.collaborator_classname(), e))
 
     def __repr__(self):
         return "%s.%s%s" % (self._proxy.collaborator_classname(),
